# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import time
import numpy as np
import sys
import argparse

defaultPath = "."

CLI=argparse.ArgumentParser()
CLI.add_argument(
   "--device_ids",
   nargs="*",
   type=int,
   default=[]
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)

CLI.add_argument(
   "--use_balanced_class_weights",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--fit_intercept",
   action = 'store_true',
   default=False
)
CLI.add_argument(
   "--return_training_history",
   type=str,
   default=None
)

args = CLI.parse_args()
t0 = time.time()

# Load data
X_train = np.load(args.data_path + "/data/epsilon.X_train.npy")
X_test  = np.load(args.data_path + "/data/epsilon.X_test.npy")
y_train = np.load(args.data_path + "/data/epsilon.y_train.npy")
y_test  = np.load(args.data_path + "/data/epsilon.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

if args.use_gpu:
  num_threads = 256
  cpu_gpu = "GPU"
else:
  num_threads = 8
  cpu_gpu = "CPU"

if args.use_balanced_class_weights:
  class_weight = "balanced"
else:
  class_weight = None

# Create a LogisticRegression from pai4sk
from pai4sk import SupportVectorMachine

svm = SupportVectorMachine(use_gpu=args.use_gpu, num_threads=num_threads, 
                           class_weight=class_weight, device_ids=args.device_ids, 
                           regularizer=2, fit_intercept=args.fit_intercept,
                           return_training_history=args.return_training_history)

# Training
t0 = time.time()
history=svm.fit(X_train, y_train)
print("[pai4sk] {0} Training time (s):  {1:.2f}".format(cpu_gpu, time.time()-t0))


# Inference
pred = svm.predict(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred)
print("[pai4sk] {0} Accuracy: {1:.3f}".format(cpu_gpu, acc_snap))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import hinge_loss
loss_snap = hinge_loss(y_test, pred)
print("[pai4sk] {0} Hinge-loss: {1:.3f}".format(cpu_gpu, loss_snap))

if args.return_training_history is not None:
    print("[pai4sk] Training history", history)

#This is another way of of inport 

# Create a LogisticRegression from pai4sk
from pai4sk.svm import LinearSVC as SupportVectorMachine 
svm = SupportVectorMachine(use_gpu=args.use_gpu, num_threads=num_threads,
                           class_weight=class_weight, device_ids=args.device_ids,
                           fit_intercept=args.fit_intercept, loss="hinge", C=0.5, dual=True, tol=0.001,
                           return_training_history=args.return_training_history)
                           
# Training
t0 = time.time()
history=svm.fit(X_train, y_train)
print("[pai4sk.svm] {0} Training time (s):  {1:.2f}".format(cpu_gpu, time.time()-t0))


# Inference
pred = svm.predict(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import accuracy_score
acc_snap = accuracy_score(y_test, pred)
print("[pai4sk.svm] {0} Accuracy: {1:.3f}".format(cpu_gpu, acc_snap))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
from pai4sk.metrics import hinge_loss
loss_snap = hinge_loss(y_test, pred)
print("[pai4sk.svm] {0} Hinge-loss: {1:.3f}".format(cpu_gpu, loss_snap))

if args.return_training_history is not None:
    print("[pai4sk] Training history", history.training_history_)

if args.skip_sklearn:
    print('Execution using scikit-learn is skipped.')
    exit()

# Create a LogisticRegression (dual is faster for this application)
# Import sklearn's LinearSVC from pai4sk module directly
from pai4sk.svm import LinearSVCSklearn as LinearSVC
svm = LinearSVC(fit_intercept=False, dual=True, loss="hinge", C=0.5, tol=0.001)

# Training time
t0 = time.time()
svm.fit(X_train, y_train)
print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred = svm.predict(X_test)

# Evaluate accuracy on test set
acc_pai4sk       = accuracy_score(y_test, pred)
print("[sklearn] Accuracy:   {0:.3f}".format(acc_pai4sk))

# Inference
pred = svm.decision_function(X_test)

# Evaluate accuracy on test set
loss_pai4sk = hinge_loss(y_test, pred)
print("[sklearn] Hinge loss:   {0:.3f}".format(loss_pai4sk))

